@@ -23,7 +23,6 @@ which case it will just return a Poly in t, or in k(t), in which case it
 will return the fraction (fa, fd). Other variable names probably come
 from the names used in Bronstein's book.
 """
-from __future__ import print_function, division
 
 from sympy import real_roots, default_sort_key
 from sympy.abc import z
@@ -34,7 +33,7 @@ from sympy.core.power import Pow
 from sympy.core.relational import Ne
 from sympy.core.singleton import S
 from sympy.core.symbol import Symbol, Dummy
-from sympy.core.compatibility import reduce, ordered, range
+from sympy.core.compatibility import reduce, ordered
 from sympy.integrals.heurisch import _symbols
 
 from sympy.functions import (acos, acot, asin, atan, cos, cot, exp, log,
@@ -114,7 +113,7 @@ def integer_powers(exprs):
     return sorted(iter(newterms.items()), key=lambda item: item[0].sort_key())
 
 
-class DifferentialExtension(object):
+class DifferentialExtension:
     """
     A container for all the information relating to a differential extension.
 
@@ -509,8 +508,8 @@ class DifferentialExtension(object):
 
                     if const or len(ans) > 1:
                         rad = Mul(*[term**(power/n) for term, power in ans])
-                        self.newf = self.newf.xreplace(dict((exp(p*exparg),
-                            exp(const*p)*rad) for exparg, p in others))
+                        self.newf = self.newf.xreplace({exp(p*exparg):
+                            exp(const*p)*rad for exparg, p in others})
                         self.newf = self.newf.xreplace(dict(list(zip(reversed(self.T),
                             reversed([f(self.x) for f in self.Tfuncs])))))
                         restart = True
@@ -539,7 +538,7 @@ class DifferentialExtension(object):
                     i = Symbol('i')
                 self.Tfuncs += [Lambda(i, exp(arg.subs(self.x, i)))]
                 self.newf = self.newf.xreplace(
-                        dict((exp(exparg), self.t**p) for exparg, p in others))
+                        {exp(exparg): self.t**p for exparg, p in others})
                 new_extension = True
 
         if restart:
@@ -726,7 +725,7 @@ def update_sets(seq, atoms, func):
     return list(s)
 
 
-class DecrementLevel(object):
+class DecrementLevel:
     """
     A context manager for decrementing the level of a DifferentialExtension.
     """
@@ -769,14 +768,10 @@ def gcdex_diophantine(a, b, c):
     # XXX: Bettter name?
 
     s, g = a.half_gcdex(b)
-    q = c.exquo(g)  # Inexact division means c is not in (a, b)
-    s = q*s
-
-    if not s.is_zero and b.degree() >= b.degree():
-        q, s = s.div(b)
-
+    s *= c.exquo(g)  # Inexact division means c is not in (a, b)
+    if s and s.degree() >= b.degree():
+        _, s = s.div(b)
     t = (c - s*a).exquo(b)
-
     return (s, t)
 
 
@@ -895,7 +890,7 @@ def derivation(p, DE, coefficientD=False, basic=False):
         if basic:
             r += d.as_expr()*pv.diff(v)
         else:
-            r += (d*pv.diff(v)).as_poly(t)
+            r += (d.as_expr()*pv.diff(v).as_expr()).as_poly(t)
 
     if basic:
         r = cancel(r)
@@ -912,7 +907,7 @@ def get_case(d, t):
     Returns one of {'exp', 'tan', 'base', 'primitive', 'other_linear',
     'other_nonlinear'}.
     """
-    if not d.has(t):
+    if not d.expr.has(t):
         if d.is_one:
             return 'base'
         return 'primitive'
@@ -945,7 +940,7 @@ def splitfactor(p, DE, coefficientD=False, z=None):
     if p.is_zero:
         return (p, One)
 
-    if not p.has(DE.t):
+    if not p.expr.has(DE.t):
         s = p.as_poly(*kinv).gcd(Dp.as_poly(*kinv)).as_poly(DE.t)
         n = p.exquo(s)
         return (n, s)
@@ -1139,8 +1134,10 @@ def laurent_series(a, d, F, n, DE):
         Q = Pa.quo(Pd)
         for i in range(0, j + 1):
             Q = Q.subs(Z[i], V[i])
-        Dha = hd*derivation(ha, DE, basic=True) + ha*derivation(hd, DE, basic=True)
-        Dha += hd*derivation(ha, DE_new, basic=True) + ha*derivation(hd, DE_new, basic=True)
+        Dha = (hd*derivation(ha, DE, basic=True).as_poly(DE.t)
+             + ha*derivation(hd, DE, basic=True).as_poly(DE.t)
+             + hd*derivation(ha, DE_new, basic=True).as_poly(DE.t)
+             + ha*derivation(hd, DE_new, basic=True).as_poly(DE.t))
         Dhd = Poly(j + 1, DE.t)*hd**2
         ha, hd = Dha, Dhd
 
@@ -1286,7 +1283,7 @@ def residue_reduce(a, d, DE, z=None, invert=True):
                 inv, coeffs = h_lc.as_poly(z, field=True).invert(s), [S.One]
 
                 for coeff in h.coeffs()[1:]:
-                    L = reduced(inv*coeff, [s])[1]
+                    L = reduced(inv*coeff.as_poly(inv.gens), [s])[1]
                     coeffs.append(L.as_expr())
 
                 h = Poly(dict(list(zip(h.monoms(), coeffs))), DE.t)
@@ -1306,8 +1303,8 @@ def residue_reduce_to_basic(H, DE, z):
     i = Dummy('i')
     s = list(zip(reversed(DE.T), reversed([f(DE.x) for f in DE.Tfuncs])))
 
-    return sum((RootSum(a[0].as_poly(z), Lambda(i, i*log(a[1].as_expr()).subs(
-        {z: i}).subs(s))) for a in H))
+    return sum(RootSum(a[0].as_poly(z), Lambda(i, i*log(a[1].as_expr()).subs(
+        {z: i}).subs(s))) for a in H)
 
 
 def residue_reduce_derivation(H, DE, z):
@@ -1319,8 +1316,8 @@ def residue_reduce_derivation(H, DE, z):
     """
     # TODO: verify that this is correct for multiple extensions
     i = Dummy('i')
-    return S(sum((RootSum(a[0].as_poly(z), Lambda(i, i*derivation(a[1],
-        DE).as_expr().subs(z, i)/a[1].as_expr().subs(z, i))) for a in H)))
+    return S(sum(RootSum(a[0].as_poly(z), Lambda(i, i*derivation(a[1],
+        DE).as_expr().subs(z, i)/a[1].as_expr().subs(z, i))) for a in H))
 
 
 def integrate_primitive_polynomial(p, DE):
@@ -1337,11 +1334,11 @@ def integrate_primitive_polynomial(p, DE):
     Zero = Poly(0, DE.t)
     q = Poly(0, DE.t)
 
-    if not p.has(DE.t):
+    if not p.expr.has(DE.t):
         return (Zero, p, True)
 
     while True:
-        if not p.has(DE.t):
+        if not p.expr.has(DE.t):
             return (q, p, True)
 
         Dta, Dtb = frac_in(DE.d, DE.T[DE.level - 1])
@@ -1574,7 +1571,7 @@ def integrate_nonlinear_no_specials(a, d, DE, z=None):
         DE, z).as_expr() + r[0].as_expr()/r[1].as_expr()).as_poly(DE.t)
     q1, q2 = polynomial_reduce(p, DE)
 
-    if q2.has(DE.t):
+    if q2.expr.has(DE.t):
         b = False
     else:
         b = True
@@ -1744,7 +1741,7 @@ def risch_integrate(f, x, extension=None, handle_first='log',
 
     result = S.Zero
     for case in reversed(DE.cases):
-        if not fa.has(DE.t) and not fd.has(DE.t) and not case == 'base':
+        if not fa.expr.has(DE.t) and not fd.expr.has(DE.t) and not case == 'base':
             DE.decrement_level()
             fa, fd = frac_in((fa, fd), DE.t)
             continue
