@@ -1,11 +1,13 @@
-# -*- coding: utf-8 -*-
 """
 This module contains functions for the computation
-of Euclidean, generalized Sturmian and (modified) subresultant
-polynomial remainder sequences (prs's).
+of Euclidean, (generalized) Sturmian, (modified) subresultant
+polynomial remainder sequences (prs's) of two polynomials;
+included are also three functions for the computation of the
+resultant of two polynomials.
 
-The pseudo-remainder function prem() of sympy is _not_ used
-by any of the functions in the module.
+Except for the function res_z(), which computes the resultant
+of two polynomials, the pseudo-remainder function prem()
+of sympy is _not_ used by any of the functions in the module.
 
 Instead of prem() we use the function
 
@@ -13,10 +15,13 @@ rem_z().
 
 Included is also the function quo_z().
 
+An explanation of why we avoid prem() can be found in the
+references stated in the docstring of rem_z().
+
 1. Theoretical background:
 ==========================
-Consider the polynomials f, g ∈ Z[x] of degrees deg(f) = n and
-deg(g) = m with n ≥ m.
+Consider the polynomials f, g in Z[x] of degrees deg(f) = n and
+deg(g) = m with n >= m.
 
 Definition 1:
 =============
@@ -34,20 +39,20 @@ degree difference between any two consecutive polynomials is 1;
 otherwise, it called incomplete.
 
 It is understood that f, g belong to the sequences mentioned in
-the two definitions.
+the two definitions above.
 
 1A. Euclidean and subresultant prs's:
 =====================================
 The subresultant prs of f, g is a sequence of polynomials in Z[x]
 analogous to the Euclidean prs, the sequence obtained by applying
-on f, g Euclid’s algorithm for polynomial greatest common divisors
+on f, g Euclid's algorithm for polynomial greatest common divisors
 (gcd) in Q[x].
 
 The subresultant prs differs from the Euclidean prs in that the
 coefficients of each polynomial in the former sequence are determinants
 --- also referred to as subresultants --- of appropriately selected
-sub-matrices of sylvester1(f, g, x), Sylvester’s matrix of 1840 of
-dimensions (n + m) × (n + m).
+sub-matrices of sylvester1(f, g, x), Sylvester's matrix of 1840 of
+dimensions (n + m) * (n + m).
 
 Recall that the determinant of sylvester1(f, g, x) itself is
 called the resultant of f, g and serves as a criterion of whether
@@ -59,43 +64,52 @@ determinant of sylvester(f, g, x, 1); instead, it returns
 the last member of the subresultant prs of f, g, multiplied
 (if needed) by an appropriate power of -1; see the caveat below.
 
+In this module we use three functions to compute the
+resultant of f, g:
+a) res(f, g, x) computes the resultant by evaluating
+the determinant of sylvester(f, g, x, 1);
+b) res_q(f, g, x) computes the resultant recursively, by
+performing polynomial divisions in Q[x] with the function rem();
+c) res_z(f, g, x) computes the resultant recursively, by
+performing polynomial divisions in Z[x] with the function prem().
+
 Caveat: If Df = degree(f, x) and Dg = degree(g, x), then:
 
 resultant(f, g, x) = (-1)**(Df*Dg) * resultant(g, f, x).
 
-For complete prs’s the sign sequence of the Euclidean prs of f, g
+For complete prs's the sign sequence of the Euclidean prs of f, g
 is identical to the sign sequence of the subresultant prs of f, g
 and the coefficients of one sequence  are easily computed from the
 coefficients of the  other.
 
-For incomplete prs’s the polynomials in the subresultant prs, generally
+For incomplete prs's the polynomials in the subresultant prs, generally
 differ in sign from those of the Euclidean prs, and --- unlike the
-case of complete prs’s --- it is not at all obvious how to compute
+case of complete prs's --- it is not at all obvious how to compute
 the coefficients of one sequence from the coefficients of the  other.
 
 1B. Sturmian and modified subresultant prs's:
 =============================================
-For the same polynomials f, g ∈ Z[x] mentioned above, their ``modified''
+For the same polynomials f, g in Z[x] mentioned above, their ``modified''
 subresultant prs is a sequence of polynomials similar to the Sturmian
-prs, the sequence obtained by applying in Q[x] Sturm’s algorithm on f, g.
+prs, the sequence obtained by applying in Q[x] Sturm's algorithm on f, g.
 
 The two sequences differ in that the coefficients of each polynomial
 in the modified subresultant prs are the determinants --- also referred
 to as modified subresultants --- of appropriately selected  sub-matrices
-of sylvester2(f, g, x), Sylvester’s matrix of 1853 of dimensions 2n × 2n.
+of sylvester2(f, g, x), Sylvester's matrix of 1853 of dimensions 2n x 2n.
 
 The determinant of sylvester2 itself is called the modified resultant
 of f, g and it also can serve as a criterion of whether the two
 polynomials have common roots or not.
 
-For complete prs’s the  sign sequence of the Sturmian prs of f, g is
+For complete prs's the  sign sequence of the Sturmian prs of f, g is
 identical to the sign sequence of the modified subresultant prs of
 f, g and the coefficients of one sequence  are easily computed from
 the coefficients of the other.
 
-For incomplete prs’s the polynomials in the modified subresultant prs,
+For incomplete prs's the polynomials in the modified subresultant prs,
 generally differ in sign from those of the Sturmian prs, and --- unlike
-the case of complete prs’s --- it is not at all obvious how to compute
+the case of complete prs's --- it is not at all obvious how to compute
 the coefficients of one sequence from the coefficients of the  other.
 
 As Sylvester pointed out, the coefficients of the polynomial remainders
@@ -142,11 +156,18 @@ backward_eye(deg(f))*bezout(f, g, x, method='bz')*backward_eye(deg(f))
 
 2B. Subresultant and modified subresultant prs's by
 ===================================================
-determinant evaluation:
+determinant evaluations:
 =======================
-Instead of utilizing the Sylvester matrices, we employ
-the Bezout matrix of smaller dimensions.
+We use the Sylvester matrices of 1840 and 1853 to
+compute, respectively, subresultant and modified
+subresultant polynomial remainder sequences. However,
+for large matrices this approach takes a lot of time.
 
+Instead of utilizing the Sylvester matrices, we can
+employ the Bezout matrix which is of smaller dimensions.
+
+subresultants_sylv(f, g, x)
+modified_subresultants_sylv(f, g, x)
 subresultants_bezout(f, g, x)
 modified_subresultants_bezout(f, g, x)
 
@@ -155,13 +176,13 @@ modified_subresultants_bezout(f, g, x)
 All three functions in this section evaluate one determinant
 per remainder polynomial; this is the determinant of an
 appropriately selected sub-matrix of sylvester1(f, g, x),
-Sylvester’s matrix of 1840.
+Sylvester's matrix of 1840.
 
 To compute the remainder polynomials the function
 subresultants_rem(f, g, x) employs rem(f, g, x).
-By contrast, the other two functions implement Van Vleck’s ideas
+By contrast, the other two functions implement Van Vleck's ideas
 of 1900 and compute the remainder polynomials by trinagularizing
-sylvester2(f, g, x), Sylvester’s matrix of 1853.
+sylvester2(f, g, x), Sylvester's matrix of 1853.
 
 
 subresultants_rem(f, g, x)
@@ -210,21 +231,28 @@ and A-M-V can be implemented utilizing either the function
 rem(f, g, x) or the function rem_z(f, g, x).
 
 For clearly historical reasons --- since the Collins-Brown-Traub
-coefficients-reduction factor β_i was not available in 1917 ---
+coefficients-reduction factor beta_i was not available in 1917 ---
 we have implemented the Pell-Gordon theorem with the function
 rem(f, g, x) and the A-M-V Theorem  with the function rem_z(f, g, x).
+
+2H. Resultants:
+===============
+res(f, g, x)
+res_q(f, g, x)
+res_z(f, g, x)
 """
 
 
 from __future__ import print_function, division
 
 from sympy import (Abs, degree, expand, eye, floor, LC, Matrix, nan, Poly, pprint)
-from sympy import (QQ, quo, rem, S, sign, simplify, summation, var, zeros)
+from sympy import (QQ, pquo, quo, prem, rem, S, sign, simplify, summation, var, zeros)
+from sympy.polys.polyerrors import PolynomialError
 
 def sylvester(f, g, x, method = 1):
     '''
-      The input polynomials f, g are in Z[x] or in Q[x].
-      Let mx = max( degree(f, x) , degree(g, x) ).
+      The input polynomials f, g are in Z[x] or in Q[x]. Let m = degree(f, x),
+      n = degree(g, x) and mx = max( m , n ).
 
       a. If method = 1 (default), computes sylvester1, Sylvester's matrix of 1840
           of dimension (m + n) x (m + n). The determinants of properly chosen
@@ -243,11 +271,11 @@ def sylvester(f, g, x, method = 1):
       ==========
       1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``On a Theorem
       by Van Vleck Regarding Sturm Sequences. Serdica Journal of Computing,
-      Vol. 7, No 4, 101–134, 2013.
+      Vol. 7, No 4, 101-134, 2013.
 
       2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Sturm Sequences
       and Modified Subresultant Polynomial Remainder Sequences.''
-      Serdica Journal of Computing, Vol. 8, No 1, 29–46, 2014.
+      Serdica Journal of Computing, Vol. 8, No 1, 29-46, 2014.
 
     '''
     # obtain degrees of polys
@@ -326,6 +354,251 @@ def sylvester(f, g, x, method = 1):
             k = k + 1
         return M
 
+def process_matrix_output(poly_seq, x):
+    """
+    poly_seq is a polynomial remainder sequence computed either by
+    (modified_)subresultants_bezout or by (modified_)subresultants_sylv.
+
+    This function removes from poly_seq all zero polynomials as well
+    as all those whose degree is equal to the degree of a preceding
+    polynomial in poly_seq, as we scan it from left to right.
+
+    """
+    L = poly_seq[:]  # get a copy of the input sequence
+    d = degree(L[1], x)
+    i = 2
+    while i < len(L):
+        d_i = degree(L[i], x)
+        if d_i < 0:          # zero poly
+            L.remove(L[i])
+            i = i - 1
+        if d == d_i:         # poly degree equals degree of previous poly
+            L.remove(L[i])
+            i = i - 1
+        if d_i >= 0:
+            d = d_i
+        i = i + 1
+
+    return L
+
+def subresultants_sylv(f, g, x):
+    """
+    The input polynomials f, g are in Z[x] or in Q[x]. It is assumed
+    that deg(f) >= deg(g).
+
+    Computes the subresultant polynomial remainder sequence (prs)
+    of f, g by evaluating determinants of appropriately selected
+    submatrices of sylvester(f, g, x, 1). The dimensions of the
+    latter are (deg(f) + deg(g)) x (deg(f) + deg(g)).
+
+    Each coefficient is computed by evaluating the determinant of the
+    corresponding submatrix of sylvester(f, g, x, 1).
+
+    If the subresultant prs is complete, then the output coincides
+    with the Euclidean sequence of the polynomials f, g.
+
+    References:
+    ===========
+    1. G.M.Diaz-Toca,L.Gonzalez-Vega: Various New Expressions for Subresultants
+    and Their Applications. Appl. Algebra in Engin., Communic. and Comp.,
+    Vol. 15, 233-266, 2004.
+
+    """
+
+    # make sure neither f nor g is 0
+    if f == 0 or g == 0:
+        return [f, g]
+
+    n = degF = degree(f, x)
+    m = degG = degree(g, x)
+
+    # make sure proper degrees
+    if n == 0 and m == 0:
+        return [f, g]
+    if n < m:
+        n, m, degF, degG, f, g = m, n, degG, degF, g, f
+    if n > 0 and m == 0:
+        return [f, g]
+
+    SR_L = [f, g]      # subresultant list
+
+    # form matrix sylvester(f, g, x, 1)
+    S = sylvester(f, g, x, 1)
+
+    # pick appropriate submatrices of S
+    # and form subresultant polys
+    j = m - 1
+
+    while j > 0:
+        Sp = S[:, :]  # copy of S
+        # delete last j rows of coeffs of g
+        for ind in range(m + n - j, m + n):
+            Sp.row_del(m + n - j)
+        # delete last j rows of coeffs of f
+        for ind in range(m - j, m):
+            Sp.row_del(m - j)
+
+        # evaluate determinants and form coefficients list
+        coeff_L, k, l = [], Sp.rows, 0
+        while l <= j:
+            coeff_L.append(Sp[ : , 0 : k].det())
+            Sp.col_swap(k - 1, k + l)
+            l += 1
+
+        # form poly and append to SP_L
+        SR_L.append(Poly(coeff_L, x).as_expr())
+        j -= 1
+
+    # j = 0
+    SR_L.append(S.det())
+
+    return process_matrix_output(SR_L, x)
+
+def modified_subresultants_sylv(f, g, x):
+    """
+    The input polynomials f, g are in Z[x] or in Q[x]. It is assumed
+    that deg(f) >= deg(g).
+
+    Computes the modified subresultant polynomial remainder sequence (prs)
+    of f, g by evaluating determinants of appropriately selected
+    submatrices of sylvester(f, g, x, 2). The dimensions of the
+    latter are (2*deg(f)) x (2*deg(f)).
+
+    Each coefficient is computed by evaluating the determinant of the
+    corresponding submatrix of sylvester(f, g, x, 2).
+
+    If the modified subresultant prs is complete, then the output coincides
+    with the Sturmian sequence of the polynomials f, g.
+
+    References:
+    ===========
+    1. A. G. Akritas,G.I. Malaschonok and P.S. Vigklas:
+    Sturm Sequences and Modified Subresultant Polynomial Remainder
+    Sequences. Serdica Journal of Computing, Vol. 8, No 1, 29--46, 2014.
+
+    """
+
+    # make sure neither f nor g is 0
+    if f == 0 or g == 0:
+        return [f, g]
+
+    n = degF = degree(f, x)
+    m = degG = degree(g, x)
+
+    # make sure proper degrees
+    if n == 0 and m == 0:
+        return [f, g]
+    if n < m:
+        n, m, degF, degG, f, g = m, n, degG, degF, g, f
+    if n > 0 and m == 0:
+        return [f, g]
+
+    SR_L = [f, g]      # modified subresultant list
+
+    # form matrix sylvester(f, g, x, 2)
+    S = sylvester(f, g, x, 2)
+
+    # pick appropriate submatrices of S
+    # and form modified subresultant polys
+    j = m - 1
+
+    while j > 0:
+        # delete last 2*j rows of pairs of coeffs of f, g
+        Sp = S[0:2*n - 2*j, :]  # copy of first 2*n - 2*j rows of S
+
+        # evaluate determinants and form coefficients list
+        coeff_L, k, l = [], Sp.rows, 0
+        while l <= j:
+            coeff_L.append(Sp[ : , 0 : k].det())
+            Sp.col_swap(k - 1, k + l)
+            l += 1
+
+        # form poly and append to SP_L
+        SR_L.append(Poly(coeff_L, x).as_expr())
+        j -= 1
+
+    # j = 0
+    SR_L.append(S.det())
+
+    return process_matrix_output(SR_L, x)
+
+def res(f, g, x):
+    """
+    The input polynomials f, g are in Z[x] or in Q[x].
+
+    The output is the resultant of f, g computed by evaluating
+    the determinant of the matrix sylvester(f, g, x, 1).
+
+    References:
+    ===========
+    1. J. S. Cohen: Computer Algebra and Symbolic Computation
+     - Mathematical Methods. A. K. Peters, 2003.
+
+    """
+    if f == 0 or g == 0:
+         raise PolynomialError("The resultant of %s and %s is not defined" % (f, g))
+    else:
+        return sylvester(f, g, x, 1).det()
+
+def res_q(f, g, x):
+    """
+    The input polynomials f, g are in Z[x] or in Q[x].
+
+    The output is the resultant of f, g computed recursively
+    by polynomial divisions in Q[x], using the function rem.
+    See Cohen's book p. 281.
+
+    References:
+    ===========
+    1. J. S. Cohen: Computer Algebra and Symbolic Computation
+     - Mathematical Methods. A. K. Peters, 2003.
+    """
+    m = degree(f, x)
+    n = degree(g, x)
+    if m < n:
+        return (-1)**(m*n) * res_q(g, f, x)
+    elif n == 0:  # g is a constant
+        return g**m
+    else:
+        r = rem(f, g, x)
+        if r == 0:
+            return 0
+        else:
+            s = degree(r, x)
+            l = LC(g, x)
+            return (-1)**(m*n) * l**(m-s)*res_q(g, r, x)
+
+def res_z(f, g, x):
+    """
+    The input polynomials f, g are in Z[x] or in Q[x].
+
+    The output is the resultant of f, g computed recursively
+    by polynomial divisions in Z[x], using the function prem().
+    See Cohen's book p. 283.
+
+    References:
+    ===========
+    1. J. S. Cohen: Computer Algebra and Symbolic Computation
+     - Mathematical Methods. A. K. Peters, 2003.
+    """
+    m = degree(f, x)
+    n = degree(g, x)
+    if m < n:
+        return (-1)**(m*n) * res_z(g, f, x)
+    elif n == 0:  # g is a constant
+        return g**m
+    else:
+        r = prem(f, g, x)
+        if r == 0:
+            return 0
+        else:
+            delta = m - n + 1
+            w = (-1)**(m*n) * res_z(g, r, x)
+            s = degree(r, x)
+            l = LC(g, x)
+            k = delta * n - m + s
+            return quo(w, l**k, x)
+
 def sign_seq(poly_seq, x):
     """
     Given a sequence of polynomials poly_seq, it returns
@@ -368,7 +641,7 @@ def bezout(p, q, x, method='bz'):
     ==========
     1. G.M.Diaz-Toca,L.Gonzalez-Vega: Various New Expressions for Subresultants
     and Their Applications. Appl. Algebra in Engin., Communic. and Comp.,
-    Vol. 15, 233–266, 2004.
+    Vol. 15, 233-266, 2004.
 
     """
     # obtain degrees of polys
@@ -433,33 +706,6 @@ def backward_eye(n):
 
     return M
 
-def process_bezout_output(poly_seq, x):
-    """
-    poly_seq is a polynomial remainder sequence computed either by
-    subresultants_bezout or by modified_subresultants_bezout.
-
-    This function removes from poly_seq all zero polynomials as well
-    as all those whose degree is equal to the degree of a previous
-    polynomial in poly_seq, as we scan it from left to right.
-
-    """
-    L = poly_seq[:]  # get a copy of the input sequence
-    d = degree(L[1], x)
-    i = 2
-    while i < len(L):
-        d_i = degree(L[i], x)
-        if d_i < 0:          # zero poly
-            L.remove(L[i])
-            i = i - 1
-        if d == d_i:         # poly degree equals degree of previous poly
-            L.remove(L[i])
-            i = i - 1
-        if d_i >= 0:
-            d = d_i
-        i = i + 1
-
-    return L
-
 def subresultants_bezout(p, q, x):
     """
     The input polynomials p, q are in Z[x] or in Q[x]. It is assumed
@@ -484,7 +730,7 @@ def subresultants_bezout(p, q, x):
     ==========
     1. G.M.Diaz-Toca,L.Gonzalez-Vega: Various New Expressions for Subresultants
     and Their Applications. Appl. Algebra in Engin., Communic. and Comp.,
-    Vol. 15, 233–266, 2004.
+    Vol. 15, 233-266, 2004.
 
     """
     # make sure neither p nor q is 0
@@ -526,10 +772,10 @@ def subresultants_bezout(p, q, x):
 
         # apply Theorem 2.1 in the paper by Toca & Vega 2004
         # to get correct signs
-        SR_L.append((int((-1)**(j*(j-1)/2)) * Poly(coeff_L, x) / F).as_expr())
+        SR_L.append(int((-1)**(j*(j-1)/2)) * (Poly(coeff_L, x) / F).as_expr())
         j = j + 1
 
-    return process_bezout_output(SR_L, x)
+    return process_matrix_output(SR_L, x)
 
 def modified_subresultants_bezout(p, q, x):
     """
@@ -555,11 +801,11 @@ def modified_subresultants_bezout(p, q, x):
     ==========
     1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Sturm Sequences
     and Modified Subresultant Polynomial Remainder Sequences.''
-    Serdica Journal of Computing, Vol. 8, No 1, 29–46, 2014.
+    Serdica Journal of Computing, Vol. 8, No 1, 29-46, 2014.
 
     2. G.M.Diaz-Toca,L.Gonzalez-Vega: Various New Expressions for Subresultants
     and Their Applications. Appl. Algebra in Engin., Communic. and Comp.,
-    Vol. 15, 233–266, 2004.
+    Vol. 15, 233-266, 2004.
 
 
     """
@@ -605,7 +851,7 @@ def modified_subresultants_bezout(p, q, x):
         SR_L.append(( Poly(coeff_L, x)).as_expr())
         j = j + 1
 
-    return process_bezout_output(SR_L, x)
+    return process_matrix_output(SR_L, x)
 
 def sturm_pg(p, q, x, method=0):
     """
@@ -647,11 +893,11 @@ def sturm_pg(p, q, x, method=0):
     ==========
     1. Pell A. J., R. L. Gordon. The Modified Remainders Obtained in Finding
     the Highest Common Factor of Two Polynomials. Annals of MatheMatics,
-    Second Series, 18 (1917), No. 4, 188–193.
+    Second Series, 18 (1917), No. 4, 188-193.
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Sturm Sequences
     and Modified Subresultant Polynomial Remainder Sequences.''
-    Serdica Journal of Computing, Vol. 8, No 1, 29–46, 2014.
+    Serdica Journal of Computing, Vol. 8, No 1, 29-46, 2014.
 
     """
     # make sure neither p nor q is 0
@@ -757,14 +1003,14 @@ def sturm_q(p, q, x):
     ==========
     1. Pell A. J., R. L. Gordon. The Modified Remainders Obtained in Finding
     the Highest Common Factor of Two Polynomials. Annals of MatheMatics,
-    Second Series, 18 (1917), No. 4, 188–193.
+    Second Series, 18 (1917), No. 4, 188-193.
 
     2 Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Sturm Sequences
     and Modified Subresultant Polynomial Remainder Sequences.''
-    Serdica Journal of Computing, Vol. 8, No 1, 29–46, 2014.
+    Serdica Journal of Computing, Vol. 8, No 1, 29-46, 2014.
 
     3. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``A Basic Result
-    on the Theory of Subresultants.'' Submitted for publication.
+    on the Theory of Subresultants.'' Serdica Journal of Computing 10 (2016), No.1, 31-48.
 
     """
     # make sure neither p nor q is 0
@@ -856,15 +1102,15 @@ def sturm_amv(p, q, x, method=0):
     References
     ==========
     1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``A Basic Result
-    on the Theory of Subresultants.'' Submitted for publication.
+    on the Theory of Subresultants.'' Serdica Journal of Computing 10 (2016), No.1, 31-48.
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``On the Remainders
     Obtained in Finding the Greatest Common Divisor of Two Polynomials.'' Serdica
-    Journal of Computing, to appear.
+    Journal of Computing 9(2) (2015), 123-138.
 
     3. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Subresultant Polynomial
     Remainder Sequences Obtained by Polynomial Divisions in Q[x] or in Z[x].''
-    Submitted for publication.
+    Serdica Journal of Computing 10 (2016), No.3-4, 197-217.
 
     """
     # compute the euclidean sequence
@@ -943,16 +1189,15 @@ def euclid_pg(p, q, x):
     References
     ==========
     1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``A Basic Result
-    on the Theory of Subresultants.'' Submitted for publication.
+    on the Theory of Subresultants.'' Serdica Journal of Computing 10 (2016), No.1, 31-48.
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``On the Remainders
     Obtained in Finding the Greatest Common Divisor of Two Polynomials.'' Serdica
-    Journal of Computing, to appear.
+    Journal of Computing 9(2) (2015), 123-138.
 
     3. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Subresultant Polynomial
     Remainder Sequences Obtained by Polynomial Divisions in Q[x] or in Z[x].''
-    Submitted for publication.
-
+    Serdica Journal of Computing 10 (2016), No.3-4, 197-217.
     """
     # compute the sturmian sequence using the Pell-Gordon (or AMV) theorem
     # with the coefficients in the prs being (in absolute value) subresultants
@@ -1012,14 +1257,14 @@ def euclid_q(p, q, x):
     ==========
     1. Pell A. J., R. L. Gordon. The Modified Remainders Obtained in Finding
     the Highest Common Factor of Two Polynomials. Annals of MatheMatics,
-    Second Series, 18 (1917), No. 4, 188–193.
+    Second Series, 18 (1917), No. 4, 188-193.
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Sturm Sequences
     and Modified Subresultant Polynomial Remainder Sequences.''
-    Serdica Journal of Computing, Vol. 8, No 1, 29–46, 2014.
+    Serdica Journal of Computing, Vol. 8, No 1, 29-46, 2014.
 
     3. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``A Basic Result
-    on the Theory of Subresultants.'' Submitted for publication.
+    on the Theory of Subresultants.'' Serdica Journal of Computing 10 (2016), No.1, 31-48.
 
     """
     # make sure neither p nor q is 0
@@ -1095,11 +1340,11 @@ def euclid_amv(f, g, x):
     References
     ==========
     1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``A Basic Result
-    on the Theory of Subresultants.'' Submitted for publication.
+    on the Theory of Subresultants.'' Serdica Journal of Computing 10 (2016), No.1, 31-48.
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Subresultant Polynomial
     remainder Sequences Obtained by Polynomial Divisions in Q[x] or in Z[x].''
-    Submitted for publication.
+    Serdica Journal of Computing 10 (2016), No.3-4, 197-217.
 
     """
     # make sure neither f nor g is 0
@@ -1169,11 +1414,11 @@ def modified_subresultants_pg(p, q, x):
     ==========
     1. Pell A. J., R. L. Gordon. The Modified Remainders Obtained in Finding
     the Highest Common Factor of Two Polynomials. Annals of MatheMatics,
-    Second Series, 18 (1917), No. 4, 188–193.
+    Second Series, 18 (1917), No. 4, 188-193.
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Sturm Sequences
     and Modified Subresultant Polynomial Remainder Sequences.''
-    Serdica Journal of Computing, Vol. 8, No 1, 29–46, 2014.
+    Serdica Journal of Computing, Vol. 8, No 1, 29-46, 2014.
 
     """
     # make sure neither p nor q is 0
@@ -1352,9 +1597,9 @@ def subresultants_pg(p, q, x):
 
     References
     ==========
-    1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ‘‘On the Remainders
-    Obtained in Finding the Greatest Common Divisor of Two Polynomials.''
-    Serdica Journal of Computing, to appear.
+    1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: "On the Remainders
+    Obtained in Finding the Greatest Common Divisor of Two Polynomials."
+    Serdica Journal of Computing 9(2) (2015), 123-138.
 
     """
     # compute the modified subresultant prs
@@ -1418,11 +1663,11 @@ def subresultants_amv_q(p, q, x):
     References
     ==========
     1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``A Basic Result
-    on the Theory of Subresultants.'' Submitted for publication.
+    on the Theory of Subresultants.'' Serdica Journal of Computing 10 (2016), No.1, 31-48.
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Subresultant Polynomial
     remainder Sequences Obtained by Polynomial Divisions in Q[x] or in Z[x].''
-    Submitted for publication.
+    Serdica Journal of Computing 10 (2016), No.3-4, 197-217.
 
     """
     # make sure neither p nor q is 0
@@ -1542,16 +1787,20 @@ def rem_z(p, q, x):
     ==========
     1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``On the Remainders
     Obtained in Finding the Greatest Common Divisor of Two Polynomials.''
-    Serdica Journal of Computing, to appear.
+    Serdica Journal of Computing, 9(2) (2015), 123-138.
 
     2. http://planetMath.org/sturmstheorem
 
     3. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``A Basic Result on
-    the Theory of Subresultants.'' Submitted for publication.
+    the Theory of Subresultants.'' Serdica Journal of Computing 10 (2016), No.1, 31-48.
 
     '''
-    delta = (degree(p, x) - degree(q, x) + 1)
-    return rem(Abs(LC(q, x))**delta  *  p, q, x)
+    if (p.as_poly().is_univariate and q.as_poly().is_univariate and
+            p.as_poly().gens == q.as_poly().gens):
+        delta = (degree(p, x) - degree(q, x) + 1)
+        return rem(Abs(LC(q, x))**delta  *  p, q, x)
+    else:
+        return prem(p, q, x)
 
 def quo_z(p, q, x):
     """
@@ -1570,8 +1819,12 @@ def quo_z(p, q, x):
     See also function rem_z(p, q, x) for additional comments and references.
 
     """
-    delta = (degree(p, x) - degree(q, x) + 1)
-    return quo(Abs(LC(q, x))**delta  *  p, q, x)
+    if (p.as_poly().is_univariate and q.as_poly().is_univariate and
+            p.as_poly().gens == q.as_poly().gens):
+        delta = (degree(p, x) - degree(q, x) + 1)
+        return quo(Abs(LC(q, x))**delta  *  p, q, x)
+    else:
+        return pquo(p, q, x)
 
 def subresultants_amv(f, g, x):
     """
@@ -1596,11 +1849,11 @@ def subresultants_amv(f, g, x):
     References
     ==========
     1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``A Basic Result
-    on the Theory of Subresultants.'' Submitted for publication.
+    on the Theory of Subresultants.'' Serdica Journal of Computing 10 (2016), No.1, 31-48.
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Subresultant Polynomial
     remainder Sequences Obtained by Polynomial Divisions in Q[x] or in Z[x].''
-    Submitted for publication.
+    Serdica Journal of Computing 10 (2016), No.3-4, 197-217.
 
     """
     # make sure neither f nor g is 0
@@ -1735,9 +1988,9 @@ def modified_subresultants_amv(p, q, x):
 
     References
     ==========
-    1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ‘‘On the Remainders
-    Obtained in Finding the Greatest Common Divisor of Two Polynomials.''
-    Serdica Journal of Computing, to appear.
+    1. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: "On the Remainders
+    Obtained in Finding the Greatest Common Divisor of Two Polynomials."
+    Serdica Journal of Computing, Serdica Journal of Computing, 9(2) (2015), 123-138.
 
     """
     # compute the subresultant prs
@@ -1802,7 +2055,7 @@ def correct_sign(deg_f, deg_g, s1, rdel, cdel):
     ==========
     Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``Sturm Sequences
     and Modified Subresultant Polynomial Remainder Sequences.''
-    Serdica Journal of Computing, Vol. 8, No 1, 29–46, 2014.
+    Serdica Journal of Computing, Vol. 8, No 1, 29-46, 2014.
 
     """
     M = s1[:, :]   # copy of matrix s1
@@ -1848,8 +2101,7 @@ def subresultants_rem(p, q, x):
     References
     ==========
     1. Akritas, A. G.:``Three New Methods for Computing Subresultant
-    Polynomial Remainder Sequences (PRS’s).'' Serdica Journal of Computing,
-    to appear.
+    Polynomial Remainder Sequences (PRS's).'' Serdica Journal of Computing 9(1) (2015), 1-26.
 
     """
     # make sure neither p nor q is 0
@@ -1913,7 +2165,7 @@ def pivot(M, i, j):
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``On a Theorem
     by Van Vleck Regarding Sturm Sequences.''
-    Serdica Journal of Computing, 7, No 4, 101–134, 2013.
+    Serdica Journal of Computing, 7, No 4, 101-134, 2013.
 
     '''
     ma = M[:, :] # copy of matrix M
@@ -2086,11 +2338,10 @@ def subresultants_vv(p, q, x, method = 0):
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``On a Theorem
     by Van Vleck Regarding Sturm Sequences.''
-    Serdica Journal of Computing, 7, No 4, 101–134, 2013.
+    Serdica Journal of Computing, 7, No 4, 101-134, 2013.
 
     3. Akritas, A. G.:``Three New Methods for Computing Subresultant
-    Polynomial Remainder Sequences (PRS’s).'' Serdica Journal of Computing,
-    to appear.
+    Polynomial Remainder Sequences (PRS's).'' Serdica Journal of Computing 9(1) (2015), 1-26.
 
     """
     # make sure neither p nor q is 0
@@ -2156,7 +2407,7 @@ def subresultants_vv(p, q, x, method = 0):
 
         # treat last row of M as poly; find its degree
         d = find_degree(M, deg_f)
-        if d == None:
+        if d is None:
             break
         exp_deg = deg_g - 1
 
@@ -2226,11 +2477,10 @@ def subresultants_vv_2(p, q, x):
 
     2. Akritas, A. G., G.I. Malaschonok and P.S. Vigklas: ``On a Theorem
     by Van Vleck Regarding Sturm Sequences.''
-    Serdica Journal of Computing, 7, No 4, 101–134, 2013.
+    Serdica Journal of Computing, 7, No 4, 101-134, 2013.
 
     3. Akritas, A. G.:``Three New Methods for Computing Subresultant
-    Polynomial Remainder Sequences (PRS’s).'' Serdica Journal of Computing,
-    to appear.
+    Polynomial Remainder Sequences (PRS's).'' Serdica Journal of Computing 9(1) (2015), 1-26.
 
     """
     # make sure neither p nor q is 0
@@ -2275,7 +2525,7 @@ def subresultants_vv_2(p, q, x):
 
         # treat last row of M as poly; find its degree
         d = find_degree(M, deg_f)
-        if d == None:
+        if d is None:
             return sr_list
         exp_deg = deg_g - 1
 
