@@ -5,16 +5,15 @@ lambda functions which can be used to calculate numerical values very fast.
 
 from __future__ import print_function, division
 
+from typing import Any, Dict
+
 import inspect
 import keyword
-import re
 import textwrap
 import linecache
 
-from types import FunctionType
-
 from sympy.core.compatibility import (exec_, is_sequence, iterable,
-    NotIterable, string_types, range, builtins, PY3)
+    NotIterable, builtins)
 from sympy.utilities.misc import filldedent
 from sympy.utilities.decorator import doctest_depends_on
 
@@ -22,13 +21,13 @@ __doctest_requires__ = {('lambdify',): ['numpy', 'tensorflow']}
 
 # Default namespaces, letting us define translations that can't be defined
 # by simple variable maps, like I => 1j
-MATH_DEFAULT = {}
-MPMATH_DEFAULT = {}
-NUMPY_DEFAULT = {"I": 1j}
-SCIPY_DEFAULT = {"I": 1j}
-TENSORFLOW_DEFAULT = {}
-SYMPY_DEFAULT = {}
-NUMEXPR_DEFAULT = {}
+MATH_DEFAULT = {}  # type: Dict[str, Any]
+MPMATH_DEFAULT = {}  # type: Dict[str, Any]
+NUMPY_DEFAULT = {"I": 1j}  # type: Dict[str, Any]
+SCIPY_DEFAULT = {"I": 1j}  # type: Dict[str, Any]
+TENSORFLOW_DEFAULT = {}  # type: Dict[str, Any]
+SYMPY_DEFAULT = {}  # type: Dict[str, Any]
+NUMEXPR_DEFAULT = {}  # type: Dict[str, Any]
 
 # These are the namespaces the lambda functions will use.
 # These are separate from the names above because they are modified
@@ -81,12 +80,12 @@ MPMATH_TRANSLATIONS = {
     "FallingFactorial": "ff",
 }
 
-NUMPY_TRANSLATIONS = {}
-SCIPY_TRANSLATIONS = {}
+NUMPY_TRANSLATIONS = {}  # type: Dict[str, str]
+SCIPY_TRANSLATIONS = {}  # type: Dict[str, str]
 
-TENSORFLOW_TRANSLATIONS = {}
+TENSORFLOW_TRANSLATIONS = {}  # type: Dict[str, str]
 
-NUMEXPR_TRANSLATIONS = {}
+NUMEXPR_TRANSLATIONS = {}  # type: Dict[str, str]
 
 # Available modules:
 MODULES = {
@@ -114,7 +113,7 @@ def _import(module, reload=False):
     other modules.
     """
     # Required despite static analysis claiming it is not used
-    from sympy.external import import_module
+    from sympy.external import import_module # noqa:F401
     try:
         namespace, namespace_default, translations, import_commands = MODULES[
             module]
@@ -170,8 +169,15 @@ _lambdify_generated_counter = 1
 @doctest_depends_on(modules=('numpy', 'tensorflow', ), python_version=(3,))
 def lambdify(args, expr, modules=None, printer=None, use_imps=True,
              dummify=False):
-    """
-    Translates a SymPy expression into an equivalent numeric function
+    """Convert a SymPy expression into a function that allows for fast
+    numeric evaluation.
+
+    .. warning::
+       This function uses ``exec``, and thus shouldn't be used on
+       unsanitized input.
+
+    Explanation
+    ===========
 
     For example, to convert the SymPy expression ``sin(x) + cos(x)`` to an
     equivalent NumPy function that numerically evaluates it:
@@ -203,75 +209,285 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
     ``x`` and SymPy functions ``sin`` and ``cos``, then converted it to an
     equivalent NumPy function ``f``, and called it on a NumPy array ``a``.
 
-    .. warning::
-       This function uses ``exec``, and thus shouldn't be used on unsanitized
-       input.
+    Parameters
+    ==========
 
-    Arguments
-    =========
+    args : List[Symbol]
+        A variable or a list of variables whose nesting represents the
+        nesting of the arguments that will be passed to the function.
 
-    The first argument of ``lambdify`` is a variable or list of variables in
-    the expression. Variable lists may be nested. Variables can be Symbols,
-    undefined functions, or matrix symbols. The order and nesting of the
-    variables corresponds to the order and nesting of the parameters passed to
-    the lambdified function. For instance,
+        Variables can be symbols, undefined functions, or matrix symbols.
 
-    >>> from sympy.abc import x, y, z
-    >>> f = lambdify([x, (y, z)], x + y + z)
-    >>> f(1, (2, 3))
-    6
+        >>> from sympy import Eq
+        >>> from sympy.abc import x, y, z
 
-    The second argument of ``lambdify`` is the expression, list of
-    expressions, or matrix to be evaluated. Lists may be nested. If the
-    expression is a list, the output will also be a list.
+        The list of variables should match the structure of how the
+        arguments will be passed to the function. Simply enclose the
+        parameters as they will be passed in a list.
 
-    >>> f = lambdify(x, [x, [x + 1, x + 2]])
+        To call a function like ``f(x)`` then ``[x]``
+        should be the first argument to ``lambdify``; for this
+        case a single ``x`` can also be used:
+
+        >>> f = lambdify(x, x + 1)
+        >>> f(1)
+        2
+        >>> f = lambdify([x], x + 1)
+        >>> f(1)
+        2
+
+        To call a function like ``f(x, y)`` then ``[x, y]`` will
+        be the first argument of the ``lambdify``:
+
+        >>> f = lambdify([x, y], x + y)
+        >>> f(1, 1)
+        2
+
+        To call a function with a single 3-element tuple like
+        ``f((x, y, z))`` then ``[(x, y, z)]`` will be the first
+        argument of the ``lambdify``:
+
+        >>> f = lambdify([(x, y, z)], Eq(z**2, x**2 + y**2))
+        >>> f((3, 4, 5))
+        True
+
+        If two args will be passed and the first is a scalar but
+        the second is a tuple with two arguments then the items
+        in the list should match that structure:
+
+        >>> f = lambdify([x, (y, z)], x + y + z)
+        >>> f(1, (2, 3))
+        6
+
+    expr : Expr
+        An expression, list of expressions, or matrix to be evaluated.
+
+        Lists may be nested.
+        If the expression is a list, the output will also be a list.
+
+        >>> f = lambdify(x, [x, [x + 1, x + 2]])
+        >>> f(1)
+        [1, [2, 3]]
+
+        If it is a matrix, an array will be returned (for the NumPy module).
+
+        >>> from sympy import Matrix
+        >>> f = lambdify(x, Matrix([x, x + 1]))
+        >>> f(1)
+        [[1]
+        [2]]
+
+        Note that the argument order here (variables then expression) is used
+        to emulate the Python ``lambda`` keyword. ``lambdify(x, expr)`` works
+        (roughly) like ``lambda x: expr``
+        (see :ref:`lambdify-how-it-works` below).
+
+    modules : str, optional
+        Specifies the numeric library to use.
+
+        If not specified, *modules* defaults to:
+
+        - ``["scipy", "numpy"]`` if SciPy is installed
+        - ``["numpy"]`` if only NumPy is installed
+        - ``["math", "mpmath", "sympy"]`` if neither is installed.
+
+        That is, SymPy functions are replaced as far as possible by
+        either ``scipy`` or ``numpy`` functions if available, and Python's
+        standard library ``math``, or ``mpmath`` functions otherwise.
+
+        *modules* can be one of the following types:
+
+        - The strings ``"math"``, ``"mpmath"``, ``"numpy"``, ``"numexpr"``,
+          ``"scipy"``, ``"sympy"``, or ``"tensorflow"``. This uses the
+          corresponding printer and namespace mapping for that module.
+        - A module (e.g., ``math``). This uses the global namespace of the
+          module. If the module is one of the above known modules, it will
+          also use the corresponding printer and namespace mapping
+          (i.e., ``modules=numpy`` is equivalent to ``modules="numpy"``).
+        - A dictionary that maps names of SymPy functions to arbitrary
+          functions
+          (e.g., ``{'sin': custom_sin}``).
+        - A list that contains a mix of the arguments above, with higher
+          priority given to entries appearing first
+          (e.g., to use the NumPy module but override the ``sin`` function
+          with a custom version, you can use
+          ``[{'sin': custom_sin}, 'numpy']``).
+
+    dummify : bool, optional
+        Whether or not the variables in the provided expression that are not
+        valid Python identifiers are substituted with dummy symbols.
+
+        This allows for undefined functions like ``Function('f')(t)`` to be
+        supplied as arguments. By default, the variables are only dummified
+        if they are not valid Python identifiers.
+
+        Set ``dummify=True`` to replace all arguments with dummy symbols
+        (if ``args`` is not a string) - for example, to ensure that the
+        arguments do not redefine any built-in names.
+
+    Examples
+    ========
+
+    >>> from sympy.utilities.lambdify import implemented_function
+    >>> from sympy import sqrt, sin, Matrix
+    >>> from sympy import Function
+    >>> from sympy.abc import w, x, y, z
+
+    >>> f = lambdify(x, x**2)
+    >>> f(2)
+    4
+    >>> f = lambdify((x, y, z), [z, y, x])
+    >>> f(1,2,3)
+    [3, 2, 1]
+    >>> f = lambdify(x, sqrt(x))
+    >>> f(4)
+    2.0
+    >>> f = lambdify((x, y), sin(x*y)**2)
+    >>> f(0, 5)
+    0.0
+    >>> row = lambdify((x, y), Matrix((x, x + y)).T, modules='sympy')
+    >>> row(1, 2)
+    Matrix([[1, 3]])
+
+    ``lambdify`` can be used to translate SymPy expressions into mpmath
+    functions. This may be preferable to using ``evalf`` (which uses mpmath on
+    the backend) in some cases.
+
+    >>> import mpmath
+    >>> f = lambdify(x, sin(x), 'mpmath')
     >>> f(1)
-    [1, [2, 3]]
+    0.8414709848078965
 
-    If it is a matrix, an array will be returned (for the NumPy module).
+    Tuple arguments are handled and the lambdified function should
+    be called with the same type of arguments as were used to create
+    the function:
 
-    >>> from sympy import Matrix
-    >>> f = lambdify(x, Matrix([x, x + 1]))
-    >>> f(1)
-    [[1]
-     [2]]
+    >>> f = lambdify((x, (y, z)), x + y)
+    >>> f(1, (2, 4))
+    3
 
-    Note that the argument order here, variables then expression, is used to
-    emulate the Python ``lambda`` keyword. ``lambdify(x, expr)`` works
-    (roughly) like ``lambda x: expr`` (see :ref:`lambdify-how-it-works` below).
+    The ``flatten`` function can be used to always work with flattened
+    arguments:
 
-    The third argument, ``modules`` is optional. If not specified, ``modules``
-    defaults to ``["scipy", "numpy"]`` if SciPy is installed, ``["numpy"]`` if
-    only NumPy is installed, and ``["math", "mpmath", "sympy"]`` if neither is
-    installed. That is, SymPy functions are replaced as far as possible by
-    either ``scipy`` or ``numpy`` functions if available, and Python's
-    standard library ``math``, or ``mpmath`` functions otherwise.
+    >>> from sympy.utilities.iterables import flatten
+    >>> args = w, (x, (y, z))
+    >>> vals = 1, (2, (3, 4))
+    >>> f = lambdify(flatten(args), w + x + y + z)
+    >>> f(*flatten(vals))
+    10
 
-    ``modules`` can be one of the following types
+    Functions present in ``expr`` can also carry their own numerical
+    implementations, in a callable attached to the ``_imp_`` attribute. This
+    can be used with undefined functions using the ``implemented_function``
+    factory:
 
-     - the strings ``"math"``, ``"mpmath"``, ``"numpy"``, ``"numexpr"``,
-       ``"scipy"``, ``"sympy"``, or ``"tensorflow"``. This uses the
-       corresponding printer and namespace mapping for that module.
-     - a module (e.g., ``math``). This uses the global namespace of the
-       module. If the module is one of the above known modules, it will also
-       use the corresponding printer and namespace mapping (i.e.,
-       ``modules=numpy`` is equivalent to ``modules="numpy"``).
-     - a dictionary that maps names of SymPy functions to arbitrary functions
-       (e.g., ``{'sin': custom_sin}``).
-     - a list that contains a mix of the arguments above, with higher priority
-       given to entries appearing first (e.g., to use the NumPy module but
-       override the ``sin`` function with a custom version, you can use
-       ``[{'sin': custom_sin}, 'numpy']``).
+    >>> f = implemented_function(Function('f'), lambda x: x+1)
+    >>> func = lambdify(x, f(x))
+    >>> func(4)
+    5
 
-    The ``dummify`` keyword argument controls whether or not the variables in
-    the provided expression that are not valid Python identifiers are
-    substituted with dummy symbols. This allows for undefined functions like
-    ``Function('f')(t)`` to be supplied as arguments. By default, the
-    variables are only dummified if they are not valid Python identifiers. Set
-    ``dummify=True`` to replace all arguments with dummy symbols (if ``args``
-    is not a string) - for example, to ensure that the arguments do not
-    redefine any built-in names.
+    ``lambdify`` always prefers ``_imp_`` implementations to implementations
+    in other namespaces, unless the ``use_imps`` input parameter is False.
+
+    Usage with Tensorflow:
+
+    >>> import tensorflow as tf
+    >>> from sympy import Max, sin, lambdify
+    >>> from sympy.abc import x
+
+    >>> f = Max(x, sin(x))
+    >>> func = lambdify(x, f, 'tensorflow')
+
+    After tensorflow v2, eager execution is enabled by default.
+    If you want to get the compatible result across tensorflow v1 and v2
+    as same as this tutorial, run this line.
+
+    >>> tf.compat.v1.enable_eager_execution()
+
+    If you have eager execution enabled, you can get the result out
+    immediately as you can use numpy.
+
+    If you pass tensorflow objects, you may get an ``EagerTensor``
+    object instead of value.
+
+    >>> result = func(tf.constant(1.0))
+    >>> print(result)
+    tf.Tensor(1.0, shape=(), dtype=float32)
+    >>> print(result.__class__)
+    <class 'tensorflow.python.framework.ops.EagerTensor'>
+
+    You can use ``.numpy()`` to get the numpy value of the tensor.
+
+    >>> result.numpy()
+    1.0
+
+    >>> var = tf.Variable(2.0)
+    >>> result = func(var) # also works for tf.Variable and tf.Placeholder
+    >>> result.numpy()
+    2.0
+
+    And it works with any shape array.
+
+    >>> tensor = tf.constant([[1.0, 2.0], [3.0, 4.0]])
+    >>> result = func(tensor)
+    >>> result.numpy()
+    [[1. 2.]
+     [3. 4.]]
+
+    Notes
+    =====
+
+    - For functions involving large array calculations, numexpr can provide a
+      significant speedup over numpy. Please note that the available functions
+      for numexpr are more limited than numpy but can be expanded with
+      ``implemented_function`` and user defined subclasses of Function. If
+      specified, numexpr may be the only option in modules. The official list
+      of numexpr functions can be found at:
+      https://numexpr.readthedocs.io/en/latest/user_guide.html#supported-functions
+
+    - In previous versions of SymPy, ``lambdify`` replaced ``Matrix`` with
+      ``numpy.matrix`` by default. As of SymPy 1.0 ``numpy.array`` is the
+      default. To get the old default behavior you must pass in
+      ``[{'ImmutableDenseMatrix':  numpy.matrix}, 'numpy']`` to the
+      ``modules`` kwarg.
+
+      >>> from sympy import lambdify, Matrix
+      >>> from sympy.abc import x, y
+      >>> import numpy
+      >>> array2mat = [{'ImmutableDenseMatrix': numpy.matrix}, 'numpy']
+      >>> f = lambdify((x, y), Matrix([x, y]), modules=array2mat)
+      >>> f(1, 2)
+      [[1]
+       [2]]
+
+    - In the above examples, the generated functions can accept scalar
+      values or numpy arrays as arguments.  However, in some cases
+      the generated function relies on the input being a numpy array:
+
+      >>> from sympy import Piecewise
+      >>> from sympy.testing.pytest import ignore_warnings
+      >>> f = lambdify(x, Piecewise((x, x <= 1), (1/x, x > 1)), "numpy")
+
+      >>> with ignore_warnings(RuntimeWarning):
+      ...     f(numpy.array([-1, 0, 1, 2]))
+      [-1.   0.   1.   0.5]
+
+      >>> f(0)
+      Traceback (most recent call last):
+          ...
+      ZeroDivisionError: division by zero
+
+      In such cases, the input should be wrapped in a numpy array:
+
+      >>> with ignore_warnings(RuntimeWarning):
+      ...     float(f(numpy.array([0])))
+      0.0
+
+      Or if numpy functionality is not required another module can be used:
+
+      >>> f = lambdify(x, Piecewise((x, x <= 1), (1/x, x > 1)), "math")
+      >>> f(0)
+      0
 
     .. _lambdify-how-it-works:
 
@@ -514,171 +730,6 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
     (say, NumPy arrays).** Remember that by default, if the ``module``
     argument is not provided, ``lambdify`` creates functions using the NumPy
     and SciPy namespaces.
-
-    Examples
-    ========
-
-    >>> from sympy.utilities.lambdify import implemented_function
-    >>> from sympy import sqrt, sin, Matrix
-    >>> from sympy import Function
-    >>> from sympy.abc import w, x, y, z
-
-    >>> f = lambdify(x, x**2)
-    >>> f(2)
-    4
-    >>> f = lambdify((x, y, z), [z, y, x])
-    >>> f(1,2,3)
-    [3, 2, 1]
-    >>> f = lambdify(x, sqrt(x))
-    >>> f(4)
-    2.0
-    >>> f = lambdify((x, y), sin(x*y)**2)
-    >>> f(0, 5)
-    0.0
-    >>> row = lambdify((x, y), Matrix((x, x + y)).T, modules='sympy')
-    >>> row(1, 2)
-    Matrix([[1, 3]])
-
-    ``lambdify`` can be used to translate SymPy expressions into mpmath
-    functions. This may be preferable to using ``evalf`` (which uses mpmath on
-    the backend) in some cases.
-
-    >>> import mpmath
-    >>> f = lambdify(x, sin(x), 'mpmath')
-    >>> f(1)
-    0.8414709848078965
-
-    Tuple arguments are handled and the lambdified function should
-    be called with the same type of arguments as were used to create
-    the function:
-
-    >>> f = lambdify((x, (y, z)), x + y)
-    >>> f(1, (2, 4))
-    3
-
-    The ``flatten`` function can be used to always work with flattened
-    arguments:
-
-    >>> from sympy.utilities.iterables import flatten
-    >>> args = w, (x, (y, z))
-    >>> vals = 1, (2, (3, 4))
-    >>> f = lambdify(flatten(args), w + x + y + z)
-    >>> f(*flatten(vals))
-    10
-
-    Functions present in ``expr`` can also carry their own numerical
-    implementations, in a callable attached to the ``_imp_`` attribute. This
-    can be used with undefined functions using the ``implemented_function``
-    factory:
-
-    >>> f = implemented_function(Function('f'), lambda x: x+1)
-    >>> func = lambdify(x, f(x))
-    >>> func(4)
-    5
-
-    ``lambdify`` always prefers ``_imp_`` implementations to implementations
-    in other namespaces, unless the ``use_imps`` input parameter is False.
-
-    Usage with Tensorflow:
-
-    >>> import tensorflow as tf
-    >>> from sympy import Max, sin, lambdify
-    >>> from sympy.abc import x
-
-    >>> f = Max(x, sin(x))
-    >>> func = lambdify(x, f, 'tensorflow')
-
-    After tensorflow v2, eager execution is enabled by default.
-    If you want to get the compatible result across tensorflow v1 and v2
-    as same as this tutorial, run this line.
-
-    >>> tf.compat.v1.enable_eager_execution()
-
-    If you have eager execution enabled, you can get the result out
-    immediately as you can use numpy.
-
-    If you pass tensorflow objects, you may get an ``EagerTensor``
-    object instead of value.
-
-    >>> result = func(tf.constant(1.0))
-    >>> print(result)
-    tf.Tensor(1.0, shape=(), dtype=float32)
-    >>> print(result.__class__)
-    <class 'tensorflow.python.framework.ops.EagerTensor'>
-
-    You can use ``.numpy()`` to get the numpy value of the tensor.
-
-    >>> result.numpy()
-    1.0
-
-    >>> var = tf.Variable(2.0)
-    >>> result = func(var) # also works for tf.Variable and tf.Placeholder
-    >>> result.numpy()
-    2.0
-
-    And it works with any shape array.
-
-    >>> tensor = tf.constant([[1.0, 2.0], [3.0, 4.0]])
-    >>> result = func(tensor)
-    >>> result.numpy()
-    [[1. 2.]
-     [3. 4.]]
-
-    Notes
-    =====
-
-    - For functions involving large array calculations, numexpr can provide a
-      significant speedup over numpy. Please note that the available functions
-      for numexpr are more limited than numpy but can be expanded with
-      ``implemented_function`` and user defined subclasses of Function. If
-      specified, numexpr may be the only option in modules. The official list
-      of numexpr functions can be found at:
-      https://numexpr.readthedocs.io/en/latest/user_guide.html#supported-functions
-
-    - In previous versions of SymPy, ``lambdify`` replaced ``Matrix`` with
-      ``numpy.matrix`` by default. As of SymPy 1.0 ``numpy.array`` is the
-      default. To get the old default behavior you must pass in
-      ``[{'ImmutableDenseMatrix':  numpy.matrix}, 'numpy']`` to the
-      ``modules`` kwarg.
-
-      >>> from sympy import lambdify, Matrix
-      >>> from sympy.abc import x, y
-      >>> import numpy
-      >>> array2mat = [{'ImmutableDenseMatrix': numpy.matrix}, 'numpy']
-      >>> f = lambdify((x, y), Matrix([x, y]), modules=array2mat)
-      >>> f(1, 2)
-      [[1]
-       [2]]
-
-    - In the above examples, the generated functions can accept scalar
-      values or numpy arrays as arguments.  However, in some cases
-      the generated function relies on the input being a numpy array:
-
-      >>> from sympy import Piecewise
-      >>> from sympy.utilities.pytest import ignore_warnings
-      >>> f = lambdify(x, Piecewise((x, x <= 1), (1/x, x > 1)), "numpy")
-
-      >>> with ignore_warnings(RuntimeWarning):
-      ...     f(numpy.array([-1, 0, 1, 2]))
-      [-1.   0.   1.   0.5]
-
-      >>> f(0)
-      Traceback (most recent call last):
-          ...
-      ZeroDivisionError: division by zero
-
-      In such cases, the input should be wrapped in a numpy array:
-
-      >>> with ignore_warnings(RuntimeWarning):
-      ...     float(f(numpy.array([0])))
-      0.0
-
-      Or if numpy functionality is not required another module can be used:
-
-      >>> f = lambdify(x, Piecewise((x, x <= 1), (1/x, x > 1)), "math")
-      >>> f(0)
-      0
-
     """
     from sympy.core.symbol import Symbol
 
@@ -705,7 +756,7 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
     if use_imps:
         namespaces.append(_imp_namespace(expr))
     # Check for dict before iterating
-    if isinstance(modules, (dict, string_types)) or not hasattr(modules, '__iter__'):
+    if isinstance(modules, (dict, str)) or not hasattr(modules, '__iter__'):
         namespaces.append(modules)
     else:
         # consistency check
@@ -837,7 +888,7 @@ def _get_namespace(m):
     """
     This is used by _lambdify to parse its arguments.
     """
-    if isinstance(m, string_types):
+    if isinstance(m, str):
         _import(m)
         return MODULES[m][0]
     elif isinstance(m, dict):
@@ -885,7 +936,7 @@ def lambdastr(args, expr, printer=None, dummify=None):
         from sympy.printing.lambdarepr import lambdarepr
 
     def sub_args(args, dummies_dict):
-        if isinstance(args, string_types):
+        if isinstance(args, str):
             return args
         elif isinstance(args, DeferredVector):
             return str(args)
@@ -902,19 +953,13 @@ def lambdastr(args, expr, printer=None, dummify=None):
                 return str(args)
 
     def sub_expr(expr, dummies_dict):
-        try:
-            expr = sympify(expr).xreplace(dummies_dict)
-        except Exception:
-            if isinstance(expr, DeferredVector):
-                pass
-            elif isinstance(expr, dict):
-                k = [sub_expr(sympify(a), dummies_dict) for a in expr.keys()]
-                v = [sub_expr(sympify(a), dummies_dict) for a in expr.values()]
-                expr = dict(zip(k, v))
-            elif isinstance(expr, tuple):
-                expr = tuple(sub_expr(sympify(a), dummies_dict) for a in expr)
-            elif isinstance(expr, list):
-                expr = [sub_expr(sympify(a), dummies_dict) for a in expr]
+        expr = sympify(expr)
+        # dict/tuple are sympified to Basic
+        if isinstance(expr, Basic):
+            expr = expr.xreplace(dummies_dict)
+        # list is not sympified to Basic
+        elif isinstance(expr, list):
+            expr = [sub_expr(a, dummies_dict) for a in expr]
         return expr
 
     # Transform args
@@ -953,14 +998,14 @@ def lambdastr(args, expr, printer=None, dummify=None):
     if dummify:
         args = sub_args(args, dummies_dict)
     else:
-        if isinstance(args, string_types):
+        if isinstance(args, str):
             pass
         elif iterable(args, exclude=DeferredVector):
             args = ",".join(str(a) for a in args)
 
     # Transform expr
     if dummify:
-        if isinstance(expr, string_types):
+        if isinstance(expr, str):
             pass
         else:
             expr = sub_expr(expr, dummies_dict)
@@ -1030,18 +1075,10 @@ class _EvaluatorPrinter(object):
 
         return '\n'.join(funclines) + '\n'
 
-    if PY3:
-        @classmethod
-        def _is_safe_ident(cls, ident):
-            return isinstance(ident, string_types) and ident.isidentifier() \
-                    and not keyword.iskeyword(ident)
-    else:
-        _safe_ident_re = re.compile('^[a-zA-Z_][a-zA-Z0-9_]*$')
-
-        @classmethod
-        def _is_safe_ident(cls, ident):
-            return isinstance(ident, string_types) and cls._safe_ident_re.match(ident) \
-                and not (keyword.iskeyword(ident) or ident == 'None')
+    @classmethod
+    def _is_safe_ident(cls, ident):
+        return isinstance(ident, str) and ident.isidentifier() \
+                and not keyword.iskeyword(ident)
 
     def _preprocess(self, args, expr):
         """Preprocess args, expr to replace arguments that do not map
@@ -1265,7 +1302,7 @@ def implemented_function(symfunc, implementation):
     if isinstance(symfunc, UndefinedFunction):
         kwargs = symfunc._kwargs
         symfunc = symfunc.__name__
-    if isinstance(symfunc, string_types):
+    if isinstance(symfunc, str):
         # Keyword arguments to UndefinedFunction are added as attributes to
         # the created class.
         symfunc = UndefinedFunction(
